#include "../include/org_tigris_subversion_javahl_SVNClientForPacker.h"
#include "SVNClientPackerExtension.h"
#include "Revision.h"
#include "JNIUtil.h"
#include "JNIStackElement.h"
#include "JNIStringHolder.h"
#include "svn_private_config.h"

#include <jni.h>

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    ctNative
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_ctNative
(JNIEnv *env, jobject jthis, jlong jsuper) {
  JNIEntry(SVNClientPackerExtension, ctNative);

  SVNClient* super = (SVNClient*)jsuper;

  SVNClientPackerExtension *obj = new SVNClientPackerExtension(super);
  return obj->getCppAddr();
  
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    headRevision
 * Signature: (Ljava/lang/String;)Lorg/tigris/subversion/javahl/Revision;
 */
JNIEXPORT jobject JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_headRevision
(JNIEnv *env, jobject jthis, jstring jpath) {
  JNIEntry(SVNClientPackerExtension, headRevision);
  SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
  if(cl == NULL)
    {
      JNIUtil::throwError("bad c++ this");
      return NULL;
    }
  JNIStringHolder path(jpath);
  if(JNIUtil::isExceptionThrown())
    {
      return NULL;
    }
  return cl->headRevision(jthis, path);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClient
 * Method:    rename
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;
 *             Lorg/tigris/subversion/javahl/Revision;Z)V
 */
JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_rename
  (JNIEnv *env, jobject jthis, jstring jsrcPath, jstring jdestPath, 
   jstring jmessage, jobject jrevision, jboolean jforce)
{
    JNIEntry(SVNClientPackerExtension, rename);
    SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
    if(cl == NULL)
    {
        JNIUtil::throwError(_("bad c++ this"));
        return;
    }
    Revision revision(jrevision);
    if(JNIUtil::isExceptionThrown())
    {
        return;
    }
    JNIStringHolder srcPath(jsrcPath);
    if(JNIUtil::isExceptionThrown())
    {
        return;
    }
    JNIStringHolder destPath(jdestPath);
    if(JNIUtil::isExceptionThrown())
    {
        return;
    }
    JNIStringHolder message(jmessage);
    if(JNIUtil::isExceptionThrown())
    {
        return;
    }
    cl->rename(srcPath, destPath, message, revision, jforce ? true:false);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    put
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_put
(JNIEnv *env, jobject jthis, jstring jsrcPath, jstring jdestPath, jboolean jforbid_overwrite, jstring jmessage) {
  JNIEntry(SVNClientPackerExtension, put);
  SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
  if(cl == NULL)
    {
      JNIUtil::throwError("bad c++ this");
      return 0;
    }
  JNIStringHolder srcPath(jsrcPath);
  JNIStringHolder destPath(jdestPath);
  JNIStringHolder message(jmessage);

  if(JNIUtil::isExceptionThrown())
    {
      return 0;
    }
  return cl->put(jthis, srcPath, destPath, jforbid_overwrite ? true:false, message);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    propertySet
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_propertySet
(JNIEnv *env, jobject jthis, jstring jurl, jstring jpropname, jstring jpropval, jstring jmessage) {
  JNIEntry(SVNClientPackerExtension, propertySet);
  SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
  if(cl == NULL)
    {
      JNIUtil::throwError("bad c++ this");
      return 0;
    }
  JNIStringHolder url(jurl);
  JNIStringHolder propname(jpropname);
  JNIStringHolder propval(jpropval);
  JNIStringHolder message(jmessage);

  if(JNIUtil::isExceptionThrown())
    {
      return 0;
    }
  return cl->propertySet(jthis,url,propname,propval,message);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    properties
 * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;)[Lorg/tigris/subversion/javahl/PropertyData;
 */
JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_properties
  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevision)
{
  JNIEntry(SVNClientPackerExtension, properties);
  SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
  if(cl == NULL)
    {
      JNIUtil::throwError("bad c++ this");
      return 0;
    }
    JNIStringHolder path(jpath);
    if(JNIUtil::isExceptionThrown())
    {
        return NULL;
    }
    Revision revision(jrevision);
    if(JNIUtil::isExceptionThrown())
    {
        return 0;
    }

    return cl->properties(jthis, path, revision);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    propertyRemove
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_propertyRemove
(JNIEnv *env, jobject jthis, jstring jurl, jstring jpropname, jstring jmessage)
{
  JNIEntry(SVNClientPackerExtension, propertyRemove);
    SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
  if(cl == NULL)
    {
      JNIUtil::throwError("bad c++ this");
      return 0;
    }
  JNIStringHolder url(jurl);
  JNIStringHolder propname(jpropname);
  JNIStringHolder message(jmessage);

  if(JNIUtil::isExceptionThrown())
    {
      return 0;
    }
  return cl->propertyRemove(jthis,url,propname,message);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    repositoryUrl
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_repositoryUrl
(JNIEnv *env, jobject jthis, jstring jpath)
{
  JNIEntry(SVNClientPackerExtension, headRevision);
  SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
  if(cl == NULL)
    {
      JNIUtil::throwError("bad c++ this");
      return NULL;
    }
  JNIStringHolder path(jpath);
  if(JNIUtil::isExceptionThrown())
    {
      return NULL;
    }
  return cl->repositoryUrl(jthis, path);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    nodeKind
 * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;)I
 */
JNIEXPORT jint JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_nodeKind
  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevision)
{
  JNIEntry(SVNClient, kind);
  SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);

    if(cl == NULL)
    {
        JNIUtil::throwError(_("bad c++ this"));
        return 0;
    }
    JNIStringHolder path(jpath);
    if(JNIUtil::isExceptionThrown())
    {
        return 0;
    }
    Revision revision(jrevision);
    if(JNIUtil::isExceptionThrown())
    {
        return 0;
    }
    return cl->kind(path, revision);
}



/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    disposeExtension
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_disposeExtension
(JNIEnv *env, jobject jthis) {
    JNIEntry(SVNClientPackerExtension, dispose);
    SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
    if(cl == NULL)
    {
        JNIUtil::throwError(_("bad c++ this"));
        return;
    }
    cl->dispose(jthis);
}

/*
 * Class:     org_tigris_subversion_javahl_SVNClientForPacker
 * Method:    finalizeExtension
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClientForPacker_finalizeExtension
(JNIEnv *env, jobject jthis) {
    JNIEntry(SVNClientPackerExtension, finalize);
    SVNClientPackerExtension *cl = SVNClientPackerExtension::getCppObject(jthis);
    if(cl != NULL)
    {
        cl->finalize();
    }
}
