/*
 * Created on 27.09.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.tigris.subversion.javahl.forpacker;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;

import org.tigris.subversion.javahl.ClientException;
import org.tigris.subversion.javahl.NodeKind;
import org.tigris.subversion.javahl.PropertyData;
import org.tigris.subversion.javahl.Revision;

import de.antivir.packer.utils.Properties;

import junit.framework.TestCase;

/**
 * @author dhoffmann
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class SVNClientTest extends TestCase {

    public static void main(String[] args) {
        junit.textui.TestRunner.run(SVNClientTest.class);
    }

    private SVNClient svn;

    private Properties properties = null;

    private String testRepoUrl = null;

    private String trunkUrl = null;

    private String username = null;

    private String password = null;

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();

        if (properties == null) {

            properties = new Properties();

            properties.loadAlongClassOf(this, "test.properties");

        }

        if (testRepoUrl == null)
            testRepoUrl = properties.getProperty("test.repo.url");
        if (username == null)
            username = properties.getProperty("test.repo.username");
        if (password == null)
            password = properties.getProperty("test.repo.password");

        if (svn == null) {
            try {
                svn = new SVNClient();
            } catch (UnsatisfiedLinkError e) {
                if (e.getMessage().matches(".*__gxx_personality_v0.*")) {
                    System.err
                            .println("Caught UnsatisfiedLinkError exception with the following message:\n\n"
                                    + e.getMessage()
                                    + "\n\n"
                                    + "To avoid this run make like this:\n\n"
                                    + "make EXTRA_LDFLAGS=\"-lstdc++\" javahl\n");
                } else {
                    throw e;
                }
            }
        }
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * Constructor for SVNClientTest.
     * 
     * @param arg0
     */
    public SVNClientTest(String arg0) {
        super(arg0);
    }

    public void testHeadRevision() throws Exception {
        //TODO Implement headRevision().
    }

    public void assertFileContent(String destUrl, Revision rev, String expected)
            throws Exception {
        byte[] buffer = svn.fileContent(destUrl, rev);

        String reallyGot = new String(buffer);

        assertEquals(expected, reallyGot);
    }

    public Revision writeToUrl(String destUrl, String expected)
            throws Exception {
        File osFile = File.createTempFile("svntest", null);

        try {
            PrintStream out = new PrintStream(new FileOutputStream(osFile));

            out.print(expected);
            out.close();

            Revision.Number beforeRev = (Revision.Number) svn
                    .headRevision(destUrl);
            long afterRev = svn.put(osFile.toString(), destUrl, "test");
            assertFileContent(destUrl, Revision.HEAD, expected);
            assertEquals(beforeRev.getNumber() + 1, afterRev);

            return new Revision.Number(afterRev);
        } finally {
            osFile.delete();
        }
    }

    public void testPut() throws Exception {
        File osFile = File.createTempFile("svntest", null);
        ;
        String destUrl = testRepoUrl + "/put-file";
        String[] expected = { "Initial version", "Second version",
                "Third version" };
        Revision[] rev = new Revision[expected.length];

        try {
            for (int i = 0; i < expected.length; ++i) {
                rev[i] = writeToUrl(destUrl, expected[i]);
            }

            for (int i = 0; i < expected.length; ++i) {
                assertFileContent(destUrl, rev[i], expected[i]);
            }
        }

        finally {
            osFile.delete();
            try {
                svn.remove(new String[] { destUrl }, "removed", false);
            } catch (ClientException e) {
                ;
            }
        }
    }

    
    public void testPropertySet(String destUrl, String propName, String expected) throws Exception {
        svn.propertySet(destUrl,propName,expected,"Set prop");
        PropertyData data = svn.propertyGet(destUrl,propName);
        
        String reallyGot = data.getValue();
        
        assertEquals(expected,reallyGot);
    }
    
    public void testPropertySet() throws Exception {
        File osFile = File.createTempFile("svntest", null);
        ;
        String destUrl = testRepoUrl + "/put-file";
        try {
            svn.remove(new String[] { destUrl }, "removed", false);
        } catch (ClientException e) {
            ;
        }

        try {
            svn.put(osFile.toString(), destUrl, "Created empty file");
            testPropertySet(destUrl,"a_property","Hallo Welt");
            testPropertySet(destUrl,"a_property","Hello World");            
        }
        
        finally {
            osFile.delete();
            try {
                svn.remove(new String[] { destUrl }, "removed", false);
            } catch (ClientException e) {
                ;
            }
        }
    }

    public void assertNodeKind(int expected, String url) throws Exception {
        int kind;
        kind = svn.nodeKind(url, Revision.HEAD);
        assertEquals(expected, kind);
    }

    public void testNodeKind() throws Exception {
        assertNodeKind(NodeKind.dir, testRepoUrl);
        assertNodeKind(NodeKind.dir, testRepoUrl + "/trunk");
        assertNodeKind(NodeKind.none, testRepoUrl + "/blah");

        try {
            writeToUrl(testRepoUrl + "/a_file", "Hallo Welt");

            assertNodeKind(NodeKind.file, testRepoUrl + "/a_file");
        } finally {
            svn.remove(new String[] { testRepoUrl + "/a_file" }, "test", true);
        }

    }

    public void testRepositoryUrl() throws Exception {
        //TODO Implement repositoryUrl().
    }

}