
/*** Includes. ***/

#include "svn_pools.h"
#include "svn_client.h"
#include "svn_string.h"
#include "svn_error.h"
#include "cl.h"
#include "svn_private_config.h"
#include "svn_cmdline.h"

#include "svn_packer_extensions.h"


/*** Code. ***/

static svn_error_t *
print_kind(const char *path,
	   svn_node_kind_t kind,
	   apr_pool_t *pool)
{
  const char* kindText;
  switch(kind) {
  case svn_node_dir:
    kindText = "dir";
    break;
  case svn_node_file:
    kindText = "file";
    break;
  case svn_node_none:
    kindText = "none";
    break;
  default:
    return svn_error_createf(SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
			     _("'%s' is not a file or directory"), path);
  }

  SVN_ERR (svn_cmdline_printf (pool,"%s %s\n",kindText, path));
  
  return SVN_NO_ERROR;
}

/* This is what we expect to get back from the print_kind_callback()
   callback */
struct kind_baton
{
  svn_error_t* err; /* OUT */
  apr_pool_t* pool; /* IN */
};

/* A callback function to print the kind for PATH */
static void
print_kind_callback (void* baton,
		     const char* path,
		     svn_node_kind_t kind)
{
  struct kind_baton* kb = baton;

  kb->err = print_kind(path,kind,kb->pool);
}

/* This implements the `svn_opt_subcommand_t' interface. */
svn_error_t *
svn_cl__kind (apr_getopt_t *os,
             void *baton,
             apr_pool_t *pool)
{
  svn_cl__opt_state_t *opt_state = ((svn_cl__cmd_baton_t *) baton)->opt_state;
  svn_client_ctx_t *ctx = ((svn_cl__cmd_baton_t *) baton)->ctx;
  apr_array_header_t *targets;
  int i;
  struct kind_baton kb;

  apr_pool_t *subpool = svn_pool_create (pool);

  SVN_ERR (svn_opt_args_to_target_array (&targets, os,
                                         opt_state->targets,
                                         &(opt_state->start_revision),
                                         &(opt_state->end_revision),
                                         FALSE, pool));

  /* Kind cannot operate on an implicit '.' so a filename is required */
  if (! targets->nelts)
    return svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR, 0, NULL);

  if (opt_state->start_revision.kind == svn_opt_revision_unspecified)
    opt_state->start_revision.kind = svn_opt_revision_head;

  for (i = 0; i < targets->nelts; i++)
    {
      const char *target = ((const char **) (targets->elts))[i];

      svn_pool_clear (subpool);
      SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));

      kb.pool = pool;
      SVN_ERR (svn_client_kind (target, &(opt_state->start_revision),
				print_kind_callback, &kb,
				ctx, subpool));
    }
  svn_pool_destroy (subpool);

  return SVN_NO_ERROR;
}
